/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "DrvI2C.h"
#include "DrvI2C_HW.h"
#include "DrvSYS.h"
#include "DrvGPIO.h"			
#include "GPIO.h"
#include "FMC.h"
/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DataLen;
#define HW_I2C_MAX_DATA_LENGHT 32
uint8_t HW_I2C_temp_val[HW_I2C_MAX_DATA_LENGHT];

typedef enum 
{
	HW_I2C_RO = 0x01,
	HW_I2C_WO = 0x02,
	HW_I2C_RW = 0x03
}HW_I2C_Cmd_Op;
/*HW_I2C command table structure*/
typedef struct{
		uint8_t Cmd;		 					/* Rev Command*/
		HW_I2C_Cmd_Op HW_I2C_Op;						/* Read/Write operation*/
		uint8_t Block_Byte_Count;				/* Only for Block data byte count */
		uint8_t *Data; 							/* SMBus has max 32 data lenght, byte_coount 1 byte only for block type*/
}HW_I2C_Data_t;

#define uP_HW_ID_code				0x00
#define uP_FW_ID_code			 	0x01
#define uP_Boot_Selection			0x02
#define Set_uP_boot_from_APROM		0x03
#define PV_APROM					0x04 
#define PV_APROM_Address			0x05 
#define UNDEF_CMD					0xFF

uint32_t u32HW_ID_code;
uint32_t u32PV_APROM[8];
uint8_t u8FW_ID_code[2];
uint8_t u8uP_Boot_Selection;
uint8_t u8Set_uP_boot_from_APROM;
uint32_t u32APROMaddress = 0;
uint8_t u8PVflag = 0;
 
HW_I2C_Data_t const HW_I2C_Cmd[]={
	{uP_HW_ID_code					,HW_I2C_RO,4,(uint8_t*)&u32HW_ID_code},					/*--------- 0 ------------*/       		
	{uP_FW_ID_code					,HW_I2C_RO,2,&u8FW_ID_code[0]},							/*--------- 1 ------------*/
	{uP_Boot_Selection				,HW_I2C_RO,1,&u8uP_Boot_Selection},
	{Set_uP_boot_from_APROM			,HW_I2C_WO,1,&u8Set_uP_boot_from_APROM},
	{PV_APROM						,HW_I2C_WO,32,(uint8_t*)u32PV_APROM},	
	{UNDEF_CMD						,HW_I2C_RW,HW_I2C_MAX_DATA_LENGHT,&HW_I2C_temp_val[0]},	/*--------- 5 ------------*/
    {PV_APROM_Address				,HW_I2C_RO,4,(uint8_t*)&u32APROMaddress},	
};

const uint8_t HW_I2C_Cmd_LookUp_Tab1[]={
	0,1,2,3,4,6,5,5		,5,5,5,5,5,5,5,5,   			/*0x00~0x0F*/
};

void Do_HW_I2C(HW_I2C_Data_t* HW_I2C_Data_Temp)
{
	switch(HW_I2C_Data_Temp->Cmd)
	{
		case Set_uP_boot_from_APROM:
			if(u8Set_uP_boot_from_APROM == 0xA5)
			{
				EnterAPROM();
				u32APROMaddress = 0;
				#ifdef DEBUG_ENABLE_LED
				O_R_LED = 1;
				#endif
			}
			break;
		case PV_APROM:
			Set_PV_flag();
			break;
	}	
}
void Read_HW_I2C(HW_I2C_Data_t* HW_I2C_Data_Temp)
{
	switch(HW_I2C_Data_Temp->Cmd)
	{
    
        
	}	
}


void Set_PV_flag(void)
{
	u8PVflag = 1;
}
void Clr_PV_flag(void)  
{
	if(u8PVflag)
	{
		if(u32APROMaddress < APROM_SIZE)
		{
			u8PVflag = 0;

			#ifdef DEBUG_ENABLE_LED
			if(Exe_PV_APROM(u32APROMaddress, u32PV_APROM))
				O_R_LED = 0;
			#else
			Exe_PV_APROM(u32APROMaddress, u32PV_APROM);
			#endif

			u32APROMaddress+=32;
		}
	}
}


HW_I2C_Data_t*	HW_I2C_Data;
/*---------------------------------------------------------------------------------------------------------*/
/*  I2C Tx Callback Function									                    		               */
/*---------------------------------------------------------------------------------------------------------*/
void I2C_Callback(uint32_t status)
{
	uint32_t u32Temp;
	switch(status)
	{
		/*Slave Transmitter Mode*/
		case 0xA8:						/* SLA+R has been received and ACK has been returned */
		case 0xB0:
			DrvI2C_SetTimeoutCounter(1,1);//Enable I2C time out
			DataLen = 0;
			Read_HW_I2C(HW_I2C_Data);
		case 0xB8:						/* DATA has been transmitted and ACK has been returned */
			if(HW_I2C_Data->Block_Byte_Count > DataLen)
			{
				DrvI2C_WriteData(HW_I2C_Data->Data[DataLen++]);
			}else
			{
				DrvI2C_WriteData(HW_I2C_Data->Data[DataLen - 1]);
			}
			DrvI2C_Ctrl(0, 0, 1, 1);					
			break;
		case 0xC0:						/* DATA has been transmitted and NACK has been returned */
		case 0xC8:						/* DATA has been transmitted and ACK has been returned */
			DrvI2C_Ctrl(0, 0, 1, 1);
			DrvI2C_SetTimeoutCounter(0,0);//Disable I2C time out			
			break;

		/*Slave Receiver Mode*/
		case 0x68:						/* SLA+W has been received and ACK has been returned */
		case 0x60:
			DrvI2C_Ctrl(0, 0, 1, 1);
			DrvI2C_SetTimeoutCounter(1,1);//Enable I2C time out
			DataLen = 0;
			break;
		case 0x80:						/* DATA has been received and ACK has been returned */
			u32Temp = DrvI2C_ReadData(); 
			if(DataLen)
			{//Data
				if(HW_I2C_Data->HW_I2C_Op == HW_I2C_RO)
				{

				}else if(HW_I2C_Data->Block_Byte_Count > DataLen)
				{				
					HW_I2C_Data->Data[DataLen++ - 1] = u32Temp;				 	
				}else if(HW_I2C_Data->Block_Byte_Count==DataLen)
				{//last byte				
				 	HW_I2C_Data->Data[DataLen++ - 1] = u32Temp;
					Do_HW_I2C(HW_I2C_Data);
				}
				DrvI2C_Ctrl(0, 0, 1, 1);					
			}else
			{//Command address
				if(u32Temp<0x10)
				{
					HW_I2C_Data = (HW_I2C_Data_t*) &HW_I2C_Cmd[HW_I2C_Cmd_LookUp_Tab1[u32Temp]];
				}else
				{
					HW_I2C_Data = (HW_I2C_Data_t*) &HW_I2C_Cmd[5];
				}
				DrvI2C_Ctrl(0, 0, 1, 1);					
				DataLen++;
				break;
			}

		case 0x88:						/* DATA has been received and NACK has been returned */
#if 0
			Rx_Data[DataLen++] = DrvI2C_ReadData();
			DrvI2C_Ctrl(0, 0, 1, 1);
			DrvI2C_SetTimeoutCounter(0,0);//Disable I2C time out
			break;
#endif
		
    	case 0xA0:                      /* STOP or Repeat START has been received */
    	    DrvI2C_Ctrl(0, 0, 1, 1);
			DrvI2C_SetTimeoutCounter(0,0);//Disable I2C time out
			break;

		default:
			
			InitI2C_HW();

	}
}

void I2C_TIMEOUT(uint32_t status)
{
	#ifdef DEBUG_ENABLE_SEMIHOST
	printf("I2C time out \n");
	printf("status = 0x%x\n", status);
	#endif
	InitI2C_HW();
}


void InitI2C_HW(void)
{
	uint16_t* Temp;
    /* Set I2C I/O */
    SYS->u32P3_MFP = (SYS->u32P3_MFP | 0x00103000) & 0xFFDFFFCF;
 	/* Open I2C and set clock = 100Kbps */
//	DrvI2C_Open(400000);    
    SYSCLK->APBCLK.I2C_EN = 1;
    SYS->IPRSTC2.I2C_RST = 1;
    SYS->IPRSTC2.I2C_RST = 0; 
    I2C->I2CON.ENS1 = 1;
    I2C->I2CLK = 5;//22118400/400000

	/* Set I2C slave addresses */
 	outpw(&I2C->I2CADDR0,0x15<<1);
	outpw(&I2C->I2CADDR1,0x15<<1);
	outpw(&I2C->I2CADDR2,0x15<<1);
	outpw(&I2C->I2CADDR3,0x15<<1);
	#ifdef DEBUG_ENABLE_SEMIHOST
	printf("I2C0 Slave address: 0x15\n");
	#endif				

	u32HW_ID_code = SYS->PDID;
	#ifdef DEBUG_ENABLE_SEMIHOST
	printf("HW ID: 0x%x\n",u32HW_ID_code);
	#endif

 	u8uP_Boot_Selection = FMC->ISPCON.BS;
	#ifdef DEBUG_ENABLE_SEMIHOST
	printf("BS: 0x%x\n",u8uP_Boot_Selection);
	#endif

	Temp = (uint16_t*)0x00000010;
	u8FW_ID_code[0] = *Temp & 0xff;
	u8FW_ID_code[1] = *Temp >> 8; 

	/* Enable I2C interrupt and set corresponding NVIC bit */
	I2C->I2CON.EI = 1;
    NVIC_SetPriority(I2C0_IRQn,1);
    NVIC_EnableIRQ(I2C0_IRQn);
		
	/* Install I2C call back function for write data to slave */
	DrvI2C_InstallCallback(I2CFUNC, I2C_Callback);

	/* Install I2C call back function for write data to slave */
	DrvI2C_InstallCallback(TIMEOUT, I2C_TIMEOUT);
		
	/* I2C as slave */
	DrvI2C_Ctrl(0, 0, 1, 1);
}

