/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "FMC.h"

#define ISP_Read        0x00
#define ISP_Program     0x21
#define ISP_PageErase   0x22
 #define APROM_SIZE		0x2000//8Kbytes

typedef enum
{
  CMD_ERR_ADDRESS	= 1,
  CMD_ERR_ERASE		= 2,
  CMD_ERR_PROGRAM	= 3,
  CMD_ERR_READ		= 4,
  CMD_ERR_VERIFY	= 5,
  CMD_ERR_flag		= 6
} E_PV_CMD_ERR;

uint32_t FMC_Go(void)
{
    FMC->ISPTRG.ISPGO = 1; 
	__ISB();
    
    if (FMC->ISPCON.ISPFF)
    {
    	FMC->ISPCON.ISPFF = 1;
    	return 1;
    }
    return 0;
}

uint32_t FMC_Erase(unsigned int address)
{
    FMC->u32ISPCMD = ISP_PageErase;
    FMC->u32ISPADR = address;
    return FMC_Go();
}

uint32_t FMC_Write(unsigned int address, unsigned int data)
{  
    FMC->u32ISPCMD = ISP_Program;
    FMC->u32ISPADR = address;
    FMC->u32ISPDAT = data;
    return FMC_Go();
}

uint32_t FMC_Read(unsigned int address, unsigned int * data)
{
    FMC->u32ISPCMD = ISP_Read;
    FMC->u32ISPADR = address;
    FMC->u32ISPDAT = 0x00000000;

    FMC->ISPTRG.ISPGO = 1; 
	__ISB();
    
    if (FMC->ISPCON.ISPFF)
    {
    	FMC->ISPCON.ISPFF = 1;
    	return 1;
    }

	*data = FMC->u32ISPDAT;
    return 0;
}
uint32_t Read_ISN(uint32_t *p32Data)
{
	  FMC->u32ISPCMD = ISP_Read;//Read
		FMC->u32ISPADR = 0x1F000;
		FMC->u32ISPDAT = 0;
		if(FMC_Go())
			return CMD_ERR_READ;
		p32Data[0] = FMC->u32ISPDAT;
		
		FMC->u32ISPADR += 4;
		FMC->u32ISPDAT = 0;
		if(FMC_Go())
			return CMD_ERR_READ;
		p32Data[1] = FMC->u32ISPDAT;
		return 0;
}
uint32_t Exe_PV_ISN(uint32_t *p32Data)
{
		FMC->u32ISPCMD = ISP_PageErase;//Erase
		FMC->u32ISPADR = 0x1F000;
		if(FMC_Go())
				return CMD_ERR_ERASE;
		
		FMC->u32ISPCMD = ISP_Program;//Program
		FMC->u32ISPDAT = p32Data[0];
		if(FMC_Go())
			return CMD_ERR_PROGRAM;

		FMC->u32ISPADR += 4;
		FMC->u32ISPDAT = p32Data[1];
		if(FMC_Go())
			return CMD_ERR_PROGRAM;	

    FMC->u32ISPCMD = ISP_Read;//Read
		FMC->u32ISPDAT = 0;
		if(FMC_Go())
			return CMD_ERR_READ;
		if(FMC->u32ISPDAT != p32Data[1])//Verify
			return CMD_ERR_VERIFY;

		FMC->u32ISPADR -= 4;
		FMC->u32ISPDAT = 0;
		if(FMC_Go())
			return CMD_ERR_READ;
		if(FMC->u32ISPDAT != p32Data[0])//Verify
			return CMD_ERR_VERIFY;
		return 0;
}
#if 1
uint32_t Exe_PV_APROM(uint32_t u32APROMaddress, uint32_t *u32PV_APROM)
{
	uint32_t j;

	if(u32APROMaddress>=APROM_SIZE)//Check address
		return CMD_ERR_ADDRESS;	

	for(j = 0;j<8;j++)
	{
		FMC->u32ISPADR = u32APROMaddress;

		if((u32APROMaddress & 0x1FF) == 0)
		{
			FMC->u32ISPCMD = ISP_PageErase;//Erase
			if(FMC_Go())
				return CMD_ERR_ERASE;
		}

    	FMC->u32ISPCMD = ISP_Program;//Program
		FMC->u32ISPDAT = u32PV_APROM[j];
		if(FMC_Go())
			return CMD_ERR_PROGRAM;

    	FMC->u32ISPCMD = ISP_Read;//Read
		FMC->u32ISPDAT = 0;
		if(FMC_Go())
			return CMD_ERR_READ;
		if(FMC->u32ISPDAT != u32PV_APROM[j])//Verify
			return CMD_ERR_VERIFY;

		u32APROMaddress += 4;						
	}
	return 0;
}
#else
uint32_t Exe_PV_APROM(uint32_t u32APROMaddress, uint32_t *u32PV_APROM)
{
	uint32_t j,k;

	if(u32APROMaddress>=APROM_SIZE)//Check address
		return CMD_ERR_ADDRESS;	
	for(j = 0;j<8;j++)
	{
		if((u32APROMaddress%512) == 0)
		{
			if(FMC_Erase(u32APROMaddress)!=0)//Erase
				return CMD_ERR_ERASE;
		}
		if(FMC_Write(u32APROMaddress, u32PV_APROM[j]))//Program
			return CMD_ERR_PROGRAM;
		if(FMC_Read(u32APROMaddress, &k))//Read
			return CMD_ERR_READ;
		if(k!=u32PV_APROM[j])//Verify
			return CMD_ERR_VERIFY;
		u32APROMaddress += 4;						
	}
	return 0;
}
#endif
uint32_t CheckSum(void)// 32bits checksum 
{
	//skip data flash
	uint32_t SUM_Result=0;
	uint32_t i , j;

	//Sum address 0x00000000 ~ (APROM_SIZE-4)
	for(i=0;i<(APROM_SIZE-4);i+=4)
	{
		if(FMC_Read(i, &j))
			return 1;
		SUM_Result+=j;
	}
	//Read last word
	if(FMC_Read(i, &j))
		return 1;		

	if(j!=SUM_Result)
		return 1;	

	return 0;
}

void EnterAPROM(void)
{
	if(CheckSum())
	{	
		return;
	}
	/* Set BS */
	FMC->ISPCON.BS = 0;//Boot from APROM
	//SYS->IPRSTC1.CPU_RST = 1;//CPU Reset
	outpw(&SCB->AIRCR, 0x05FA0004);//MCU Reset
	while(1);
}

void InitFMC(void)
{
	SYSCLK->AHBCLK.ISP_EN = 1;
	FMC->ISPCON.ISPEN = 1;
}

