/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "DrvI2C_SW.h"
#include "Driver\DrvSYS.h"
#include "Driver\DrvGPIO.h"
#define I2C_SW_SDA P12_DOUT
#define I2C_SW_CLK P13_DOUT
#define Init_I2C_SW_SDA DrvGPIO_Open(E_PORT1, E_PIN2, E_IO_OPENDRAIN)
#define Init_I2C_SW_CLK DrvGPIO_Open(E_PORT1, E_PIN3, E_IO_OPENDRAIN)
uint32_t u32_I2C_SW_Delay; 
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvI2C_Open				                                                   				   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*               u32BusClock   - [in]		I2C Bus Clock (Hz)       		                               */
/* Returns:                                                                                                */
/*               0 : Success											                    		       */
/*               							                                                               */
/* Description:                                                                                            */
/*               The maximum of I2C clock is 500kHz.													   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvI2C_SW_Open(uint32_t u32BusClock)
{
	if(u32BusClock>500000)
		u32_I2C_SW_Delay = 1;	 	
	else
		u32_I2C_SW_Delay =500000/u32BusClock;	/* Compute proper divider for I2C clock */
	Init_I2C_SW_SDA;
	Init_I2C_SW_CLK;
	I2C_SW_SDA = 1;
	I2C_SW_CLK = 1;
	return 0;
}
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvI2C_SW_Send_byte		                                                   				   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*               u8Data   - [in]		Data					       		                               */
/* Returns:                                                                                                */
/*               0 : ACK												                    		       */
/*               1 : NACK					                                                               */
/* Description:                                                                                            */
/*               																						   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvI2C_SW_Send_byte(uint8_t u8Data)
{
	uint32_t u32count;
	for(u32count=0;u32count<8;u32count++)
	{
		I2C_SW_SDA = u8Data>>(7-u32count);
		DrvSYS_Delay(u32_I2C_SW_Delay);
		I2C_SW_CLK = 1;
		DrvSYS_Delay(u32_I2C_SW_Delay);	
		I2C_SW_CLK = 0;
	}
	I2C_SW_SDA = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
    u32count = I2C_SW_SDA;
	I2C_SW_CLK = 0;

	return u32count;	
}
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvI2C_SW_Send				                                                   				   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*               u8Address   - [in]		I2C slave address		       		                               */
/*               p8Data      - [in]		data address			       		                               */
/*               u32bytesize - [in]		data length 			       		                               */
/* Returns:                                                                                                */
/*               data length							                    		       		           */
/*               							                                                               */
/* Description:                                                                                            */
/*               																						   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvI2C_SW_Send(uint8_t u8Address, uint8_t* p8Data, uint32_t u32ByteSize)
{
	uint32_t u32count = 0;
	if(u32ByteSize == 0)
		return 0;
	/*Start*/
	I2C_SW_SDA = 1;
	I2C_SW_CLK = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_SDA = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	/*Address*/
	if(DrvI2C_SW_Send_byte(u8Address<<1))
		goto I2C_SW_Stop_Send;
	/*Data*/
	while(u32count<u32ByteSize)
	{
		if(DrvI2C_SW_Send_byte(*(p8Data+u32count++)))
			goto I2C_SW_Stop_Send;		
	}
I2C_SW_Stop_Send:
	I2C_SW_SDA = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_SDA = 1;

	return u32count;	
}
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvI2C_SW_Get_byte			                                                   				   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*               u32Ack   - [in]		ACK/NACK				       		                               */
/* Returns:                                                                                                */
/*               Data													                    		       */
/*               							                                                               */
/* Description:                                                                                            */
/*               																						   */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvI2C_SW_Get_byte(uint32_t u32Ack)
{
	uint32_t u32count;
	uint8_t u8Data=0;

	
	for(u32count=0;u32count<8;u32count++)
	{
		DrvSYS_Delay(u32_I2C_SW_Delay);
		I2C_SW_CLK = 1;
		DrvSYS_Delay(u32_I2C_SW_Delay);
		u8Data |= I2C_SW_SDA << (7-u32count);	
		I2C_SW_CLK = 0;
	}
	I2C_SW_SDA = u32Ack;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 1;
    u32count = I2C_SW_SDA;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 0;
	I2C_SW_SDA = 1;


	return u8Data;	
}
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvI2C_SW_Get				                                                   				   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*               u8Address   - [in]		I2C slave address		       		                               */
/*               p8Data      - [in]		data address			       		                               */
/*               u32bytesize - [in]		data length 			       		                               */
/* Returns:                                                                                                */
/*               data length							                    		       		           */
/*               							                                                               */
/* Description:                                                                                            */
/*               																						   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvI2C_SW_Get(uint8_t u8Address, uint8_t* p8Data, uint32_t u32ByteSize)
{
	uint32_t u32count = 0;
	if(u32ByteSize == 0)
		return 0;
	/*Start*/
	I2C_SW_SDA = 1;
	I2C_SW_CLK = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_SDA = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	/*Address*/
	if(DrvI2C_SW_Send_byte((u8Address<<1)|1))
		goto I2C_SW_Stop_Get;
	/*Data*/
	while(u32count<(u32ByteSize-1))
	{
		*(p8Data+u32count++) = DrvI2C_SW_Get_byte(0);		
	}
	*(p8Data+u32count++) = DrvI2C_SW_Get_byte(1);
I2C_SW_Stop_Get:
	I2C_SW_SDA = 0;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_CLK = 1;
	DrvSYS_Delay(u32_I2C_SW_Delay);
	I2C_SW_SDA = 1;

	return u32count;	
}


