/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/*Interrupt setting
GPIO scan (P0P1 & Timer2) = HW I2C > WDT > SW I2C (Timer3) 

P0P1 > Timer2 = I2C > WDT >Timer3
*/
#include <stdio.h>
#include "DrvGPIO.h"
#include "DrvSYS.h"
#include "DrvI2C_HW.h"
#include "FMC.h"
#include "GPIO.h"

#define EnableWDT 0
/*--------------------------------------------------------------------------------*/
/*  Detect 10Khz 			                                       			      */
/*--------------------------------------------------------------------------------*/
#if EnableWDT
uint32_t Detect10KHz(void)
{
		
	uint32_t Temp;
	NVIC_DisableIRQ(WDT_IRQn);
	SysTick->CTRL = 0x04;
    SysTick->LOAD = 0xFFFFFF;
    SysTick->VAL  = (0x00);

	SYSCLK->CLKSEL1.WDT_S = 0x03;
	SYSCLK->APBCLK.WDT_EN = 1;

	__disable_irq();
	outpw((uint32_t)&WDT->WTCR,  0x0C9);
	while(WDT->WTCR.WTIF==0);//Delay 19*(1/10000) = 1.9ms
	outpw((uint32_t)&WDT->WTCR,  0x0C9);
	SysTick->CTRL = 0x05;
	while(WDT->WTCR.WTIF==0);//Delay 19*(1/10000) = 1.9ms
 	Temp = 0xFFFFFF-SysTick->VAL;
	__enable_irq();
	
	outpw((uint32_t)&WDT->WTCR,  0x3CB);//Enable WDT Reset
	//outpw((uint32_t)&WDT->WTCR,  0x3C9);
	NVIC_SetPriority(WDT_IRQn,2);
	NVIC_ClearPendingIRQ(WDT_IRQn);
	NVIC_EnableIRQ(WDT_IRQn);

	return ((__IRC22M*19)/Temp);//(16+3)
}
uint32_t u32WDTbase;
uint32_t u32WDTcount1S;
uint8_t u8WDTcounter;
uint8_t u8WDTcounter1;
uint8_t u8WDTflag=1;

void WDT_IRQHandler(void)
{
#ifdef WakeUpFlow
 	WakeUpFlow;
#endif
	if(u8WDTflag)
	{
		WDT->WTCR.WTR = 1;
		u8WDTflag = 0;
	}
    WDT->WTCR.WTIF = 1;         /* Clear the WDT INT Flag */

	u32WDTcount1S+=1024;

	if(u32WDTcount1S >= u32WDTbase)
	{
		u32WDTcount1S-= u32WDTbase;
		u8WDTcounter++;
		u8WDTcounter1++;

		#ifdef DEBUG_ENABLE_SEMIHOST
	 	u32WDTtest++;
		printf("WDT => %d\n",u32WDTtest);
		#endif		
	}
}
#endif

void EnterIdleMode(void)
{
#ifdef WakeUpFlow    
	SYSCLK->CLKDIV.HCLK_N = 2;		
#endif    
	__WFI();	
}



/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function									                                           			   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
	uint32_t u32Temp;

	UNLOCKREG();
	SYS->BODCR.LVR_EN = 1;//Enable LVR	

	InitFMC();// Initial FMC function

	if(SYS->u32RSTSRC & 0xA0)//Software reset : CPU and MCU reset	
	{//Debug mode entry
		SYS->u32RSTSRC = 0xA0;// Cleared by LDROM 
	}
	else// LVR, BOR, WDT, POR or Reset pin
	{
		SYS->u32RSTSRC = 0x03;// Clear POR and Reset flag at LDROM
		EnterAPROM();
	}

	InitI2C_HW();
 	#ifdef DEBUG_ENABLE_SEMIHOST
	printf("Finish I2C_HW init\n");
	#endif

	while(1)
	{
#if EnableWDT       
		if(u8WDTcounter >= 10)//Defualt value of u8WDTcount10S is 10
		{			
			u8WDTcounter = 0;
			u32Temp = Detect10KHz();//Detect internal 10Khz Oscillator
			__disable_irq();
			u32WDTbase = u32Temp;
			__enable_irq();
			#ifdef DEBUG_ENABLE_SEMIHOST
			printf("Detect 10khz = %d\n",u32WDTbase);
			#endif		
		}
		if(u8WDTcounter1 >= 20)
		{
			EnterAPROM();	
            u8WDTcounter1 = 0;
		}
		u8WDTflag = 1;        
#endif
 		Clr_PV_flag();
		EnterIdleMode();
		Clr_PV_flag();
	}
}

